local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local GetPowerRegen = GetPowerRegen
local GetTime = GetTime
local UnitPowerMax = UnitPowerMax
local math = math
local select = select
local string = string
local tostring = tostring

a.Rotations = {}

local function t15RageBump(amount)
  if c.WearingSet(4, "ProtT15") 
    and s.Debuff(c.GetID("Demoralizing Shout")) then
    
    a.Rage = a.Rage + 1.5 * amount
  else
    a.Rage = a.Rage + amount
  end
end

a.smashPending = 0
a.smashTime = 0

local monitorSmashPending = function(info) 
  if c.InfoMatches(info, "Colossus Smash") then	
    a.smashTime = GetTime()
	a.smashPending = 1
  end
end

local monitorSmashApplied = function(spellID)
  if c.IdMatches(spellID, "Colossus Smash") then
    a.smashPending = 0
  end
end

function a.PreFlash()
  if a.smashPending > 0 then
    a.smashPending = GetTime() - a.smashTime
	if a.smashPending >= 20 then
      a.smashPending = 0
    end
  end
  
  a.freeExecute = false
  a.CSIsUsable = select(1, s.UsableSpell("Colossus Smash")) and a.smashPending == 0
  a.ExeIsUsable = select(1,s.UsableSpell("Execute"))
  a.MSIsUsable = select(1, s.UsableSpell("Mortal Strike")) and c.GetCooldown("Mortal Strike") == 0
  a.Rage = c.GetPower(0)
  a.InExecute = s.HealthPercent() <= 20 and a.Rage > 30 and a.ExeIsUsable
  if c.HasBuff("Sudden Death") then
     a.freeExecute = a.ExeIsUsable
  elseif a.ExeIsUsable then
     a.freeExecute = true
  end
  a.Enraged = c.HasBuff("Enrage", true)
  if c.IsQueued("Berserker Rage") then
    a.Rage = a.Rage + 10
    a.Enraged = true
  end
  if c.IsQueued("Charge") then
    t15RageBump(20)
  end   
  if c.IsQueued("Bloodthirst", "Mortal Strike") then
    a.Rage = a.Rage + 10
  elseif c.IsQueued("Battle Shout", "Commanding Shout") then
    a.Rage = a.Rage + 20
  elseif c.IsQueued("Shield Slam") then
    if s.Buff(c.GetID("Sword and Board"), "player") then
      t15RageBump(25)
    else
      t15RageBump(20)
    end
  elseif c.IsQueued("Revenge") then
    t15RageBump(15)
  end
  local max = UnitPowerMax("player")
  a.Rage = math.min(max, a.Rage)
  a.EmptyRage = max - a.Rage
  
  if c.IsCasting("Victory Rush", "Impending Victory") then
    a.VictoriousDuration = 0
  else
    a.VictoriousDuration = c.GetBuffDuration("Victorious")
  end
  
  a.Bloodbath = c.HasBuff("Bloodbath", false, false, true)
end

-------------------------------------------------------------------------- Arms
local tastePending = 0
a.rendAppliedTime = 0

a.Rotations.Arms = {
  Spec = 1,
  
  UsefulStats = { "Strength", "Melee Hit", "Crit", "Haste" },
  
  FlashInCombat = function()
    a.TasteStacks = c.GetBuffStack("Taste for Blood")
    if c.IsCasting("Mortal Strike") or GetTime() - tastePending < .8 then
      a.TasteStacks = a.TasteStacks + 2
    end
    
    c.FlashAll(
      "Recklessness for Arms",
      "Avatar",
      "Bloodbath",
      "Berserker Rage for Arms",
      "Heroic Leap",
      "Impending Victory for Heals, Optional",
      "Victory Rush for Heals, Optional",
--      "Enraged Regeneration",
      "Sunder Armor",
      "Pummel",
      "Disrupting Shout")
    if c.AoE then
      c.FlashAll("Cleave for Arms", "Sweeping Strikes")
      c.PriorityFlash(
        "Mortal Strike",
        "Bladestorm",
        "Shockwave",
        "Dragon Roar for Arms",
        "Thunder Clap",
        "Whirlwind for Arms",
        "Colossus Smash",
--        "Overpower AoE",
        "Shout for Rage",
        "Storm Bolt",
        "Heroic Throw")
    else
	  a.PreFlash()
      c.FlashAll("Rend" )
      c.PriorityFlash(  
        "Colossus Smash for Arms",        
        "Execute for Arms",
        "Storm Bolt for Arms",
        "Dragon Roar for Arms",
        "Shockwave",
        "Slam",
        "Mortal Strike",
        "Rend",
        "Whirlwind for Arms")
    end
  end,
  
  FlashAlways = function()
    c.FlashAll("Dps Stance", "Shout for Buff")
  end,
  
  CastSucceeded = function(info)
    if c.InfoMatches(info, "Rend") then
      if a.rendAppliedTime == 0 then
        a.rendAppliedTime = GetTime()
      end
    else
      monitorSmashPending(info)
    end
  end,
  
-- Taste for Blood, each time Rend deals damage, you gain 3 rage.
  AuraApplied = function(spellID)
    if c.IdMatches(spellID, "Rend") then
      if (GetTime() - a.rendAppliedTime) >= 17 then
        a.rendAppliedTime = 0
      end
    end
  end,
    
  ExtraDebugInfo = function()
--  print("Rend: ", s.ShowBuffs("target", "harmful"))
--	print("[CS: ", a.CSIsUsable, " CS Pending: ", a.smashPending, " Rend: ", c.HasMyDebuff("Rend", false,false), " RUse: ", s.UsableSpell("Rend"), "]")
  end,
}

-------------------------------------------------------------------------- Fury
a.CleaverPending = 0
a.CleaverDumpPending = 0

a.Rotations.Fury = {
  Spec = 2,
  
  UsefulStats = { "Strength", "Crit", "Mastery" },  -- "Haste", "Multistrike", "Versatility"},
  
  FlashInCombat = function()
    c.FlashAll(
      "Berserker Rage for Fury",
      "Recklessness for Fury",
      "Avatar",
      "Bloodbath for Fury",
      "Impending Victory for Heals, Optional",
      "Victory Rush for Heals, Optional",
      "Enraged Regeneration",
      "Sunder Armor",
      "Pummel",
      "Disrupting Shout")
    if c.AoE then
      c.FlashAll("Bloodthirst")
      c.PriorityFlash(
  --      "Dragon Roar for Fury",
  --      "Shockwave",
  --      "Bladestorm",
  --      "Bloodthirst",
  --      "Raging Blow AoE",
  --      "Whirlwind",
  --      "Shout for Rage",
  --      "Storm Bolt",
        "Heroic Throw")
    else
      c.FlashAll()
      c.PriorityFlash(
        "Berserker Rage for Fury",
        "Recklessness for Fury",
        "Execute for Fury",
        "Bloodthirst",
        "Wild Strike",
        "Raging Blow",
        "Dragon Roar for Fury",
        "Storm Bolt")
    end
  end,
  
  FlashAlways = function()
    c.FlashAll("Dps Stance", "Shout for Buff")
  end,
  
  CastSucceeded = function(info)
    if c.InfoMatches(info, "Whirlwind") then
      a.CleaverPending = GetTime()
    end
  end,
  
  AuraApplied = function(spellID)
    if c.IdMatches(spellID, "Meat Cleaver") then
      a.CleaverPending = 0
    end
  end,
  
  AuraRemoved = function(spellID)
    if c.IdMatches(spellID, "Meat Cleaver") then
      a.CleaverDumpPending = 0
    end
  end,
  
  ExtraDebugInfo = function()
  end,
}

-------------------------------------------------------------------- Protection
local uncontrolledMitigationCooldowns = {
  "Spell Reflection",
}

a.RevengeReset = 0

a.Rotations.Protection = {
  Spec = 3,
  
  UsefulStats = { "Stamina", "Strength", "Dodge", "Parry", "Tanking Hit" },
  
  FlashInCombat = function()
    c.FlashMitigationBuffs(
      1,
--      uncontrolledMitigationCooldowns,
--      c.COMMON_TANKING_BUFFS,
--      "No Mitigation if Victory Available",
--      "Demoralizing Shout",
--      "Enraged Regeneration for Prot",
--      "Shield Wall under 3 min",
--      "Last Stand",
      "Shield Wall 3+ min")
    c.FlashAll(
      "Shield Block",
      "Shield Barrier",
      "Berserker Rage for Prot",
      "Heroic Strike for Prot",
      "Taunt",
      "Sunder Armor",
      "Pummel",
      "Disrupting Shout")
    c.DelayPriorityFlash(
--      "Shockwave for Stun",
--      "Dragon's Roar for Stun",
--      "Thunder Clap for Debuff",
--      "Impending Victory for Heals",
--      "Victory Rush for Heals",
--      "Shout for Rage",
--      "Delay for Prot Rage Generator",
--      "Thunder Clap for Refresh",
--      "Devestate for Debuff unless AoE",
      "Bladestorm",
      "Shockwave",
      "Bloodbath for Prot",
      "Dragon Roar for Prot",
      "Shield Slam",
      "Execute for Prot",
      "Revenge",
      "Devestate",
      "Storm Bolt",
      "Heroic Strike for Prot"       
    )
  end,
  
--  FlashAlways = function()
--    c.FlashAll("Defensive Stance", "Shout for Buff")
--  end,
--  
--  Avoided = function(missType)
--    if missType == "DODGE" or missType == "PARRY" then
--      a.RevengeReset = GetTime()
--      c.Debug("Event", "Revenge Reset Pending")
--    end
--  end,
--  
--  CastSucceeded = function(info)
--    if c.InfoMatches(info, "Revenge") then
--      a.RevengeReset = 0
--      c.Debug("Event", "Revenge Cast")
--    end
--  end,
--  
  ExtraDebugInfo = function()
--      for i = 1, GetNumSpellTabs() do
--           local name = GetSpellTabInfo(i)
--           print(name)
--        end
    

 --           for talent = 1, 30 do
 --               print(GetSpellInfo(169685))
 --           end




--      local numTabs = GetNumTalentTabs(2)
--      DEFAULT_CHAT_FRAME:AddMessage(numTabs)
--      for x=1, numTabs do
--        DEFAULT_CHAT_FRAME:AddMessage(GetTalentTabInfo(x))
--        local numTalents = GetNumTalents(x)
--        for y=1, numTalents do
--        talentName, icon, tier, column, rank, maxRank= GetTalentInfo(x,y)
--        DEFAULT_CHAT_FRAME:AddMessage(talentName..": "..rank.."/"..maxRank)
--        end
--    end
    

    -- local nameTalent, icon, tier, column, currRank, maxRank = GetTalentInfo(9)
    -- print(nameTalent, icon, tier, column, currRank, maxRank)
    
  --  return string.format("r:%d US:%d, Damage:%s, SnB:%s, r:%.1f, AoE:%s, ck:%s", 
  --    a.Rage, 

  --    print("IE: ", a.InExecute, " ", "FE: ", a.freeExecute, " Use: ", a.ExeIsUsable)
  --    tostring(c.InDamageMode()),
  --    tostring(c.HasBuff("Sword and Board")),
  --    s.SpellCooldown(c.GetID("Revenge")),
  --    tostring(c.AoE),
  --    tostring(c.GetCooldown("Dragon Roar for Prot", nil)))
  end,
}
