local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local GetTime = GetTime
local math = math

------------------------------------------------------------------------ Common
local bothShouts = { c.GetID("Battle Shout"), c.GetID("Commanding Shout") }
local function chooseShout(z, fail)
  if fail then
    return false
  end
  
  local ap = s.Buff(c.ATTACK_POWER_BUFFS, "player")
  local sta = s.Buff(c.STAMINA_BUFFS, "player")
  local battleID = c.GetID("Battle Shout")
  local commandID = c.GetID("Commanding Shout")
  if s.MyBuff(battleID, "player") then
    z.FlashID = battleID
  elseif s.MyBuff(commandID, "player") then
    z.FlashID = commandID
  elseif ap and sta then
    z.FlashID = bothShouts
  elseif ap then
    z.FlashID = commandID
  elseif sta then  
    z.FlashID = battleID
  else
    z.FlashID = bothShouts
  end
  return true
end

local function noRoomForShoutRage()
  if c.HasGlyph("Hoarse Voice") then
    return a.EmptyRage < 15
  else
    return a.EmptyRage < 30
  end
end

local function rageAfterHeroicStrike()
  if c.GetCooldown("Heroic Strike", false, 1.5) == 0 then
    return a.Rage - 30
  else
    return a.Rage
  end
end

c.AddSpell("Shout", nil, {
  ID = "Battle Shout",
  Cooldown = 60,
  CheckFirst = chooseShout
})

c.AddSpell("Shout", "for Rage", {
  ID = "Battle Shout",
  Cooldown = 60,
  CheckFirst = function(z)
    return chooseShout(z, noRoomForShoutRage())
  end
})

c.AddSpell("Shout", "for Rage unless Colossus Smash", {
  ID = "Battle Shout",
  Cooldown = 60,
  CheckFirst = function(z)
    return chooseShout(z, noRoomForShoutRage() or a.smashPending > 0) --a.Smash > 0)
  end
})

c.AddOptionalSpell("Shout", "for Buff", {
  ID = "Battle Shout",
  Cooldown = 60,
  CheckFirst = function(z)
    if not x.EnemyDetected then
      return false
    end
    
    local ap = c.RaidBuffNeeded(c.ATTACK_POWER_BUFFS)
    local sta = c.RaidBuffNeeded(c.STAMINA_BUFFS)
    local battleID = c.GetID("Battle Shout")
    local commandID = c.GetID("Commanding Shout")
    if s.MyBuff(battleID, "player") then
      z.FlashID = battleID
      return ap
    elseif s.MyBuff(commandID, "player") then
      z.FlashID = commandID
      return sta
    elseif ap and sta then
      z.FlashID = bothShouts
    elseif ap then
      z.FlashID = battleID
    elseif sta then
      z.FlashID = commandID
    else
      return false
    end
    return true
  end,
})

c.AddOptionalSpell("Dps Stance", nil, {
  Type = "form",
  ID = "Battle Stance",
  FlashID = { "Battle Stance"},
  CheckFirst = function()
    return not s.Form(c.GetID("Battle Stance"))     
  end
})

--TALENT:
--You become a whirling storm of destructive force, striking 
--all targets within 8 yards for 120% weapon damage every 1 sec 
--for 6 sec.
--During a Bladestorm, you can continue to dodge, block, and parry, 
--and are immune to movement impairing and loss of control effects. 
--However, you can only perform shout abilities.
c.AddSpell("Bladestorm", nil, {
  Melee = true,
  Cooldown = 90,
})
--TALENT"
--Sends a wave of force in a frontal cone before you, 
--causing (75 / 100 * Attack power) damage and stunning 
--all enemy targets within 10 yards for 4 sec.
--Causing damage to 3 or more targets lowers the cooldown 
--of the next Shockwave by 20 sec.
c.AddSpell("Shockwave", nil, {
  Melee = true,
  Cooldown = 40,
})

--TALENT:
--Roar ferociously, causing 126 (+ 140% of Attack power) 
--damage to all enemies within 8 yards, knocking them back 
--and knocking them down for 0.50 sec. Dragon Roar is always 
--a critical strike and ignores all armor on the target.
c.AddSpell("Dragon Roar", "for Prot", {
  Melee = true,
  Cooldown = 60,
    c.Debug("DRoar", c.HasTalent("Dragon Roar"), tostring(c.GetCooldown("Dragon Roar"))), 
  CheckLast = function()
     return c.GetCooldown("Dragon Roar") < 1
  end
})

--SPELL:
--Leap through the air towards a targeted location, slamming 
--down with destructive force to deal 1 (+ 50% of Attack power) 
--Physical damage to all enemies within 8 yards.
c.MakeMini(c.AddOptionalSpell("Heroic Leap", nil, {
  NoGCD = true,
  Cooldown = 45,
--  CheckFirst = function()
--    return a.Smash > 0 and not c.IsSolo()
--  end
}))

local function victoryHealable()
  if a.VictoriousDuration > 4 
    and c.HasGlyph("Victory Rush") 
    and not c.HasTalent("Impending Victory") then
    
    return c.GetHealthPercent("player") < 70
  else
    return c.GetHealthPercent("player") < 80
  end
end

--TALENTS:
--Instantly attack the target, causing (200% of Attack power) 
--damage and healing you for 15% of your maximum health.
--Killing an enemy that yields experience or honor resets 
--the cooldown of Impending Victory.
--Replaces Victory Rush.
c.AddSpell("Impending Victory", nil, {
  Melee = true,
  Cooldown = 30,
})

c.AddSpell("Impending Victory", "for Heals", {
  Melee = true,
  Cooldown = 30,
  CheckFirst = victoryHealable,
})

c.AddSpell("Impending Victory", "for Free", {
  Melee = true, 
  Cooldown = 30,
  CheckFirst = function()
    return a.VictoriousDuration > 0
  end
})

c.AddSpell("Impending Victory", "unless Execute", {
  Melee = true, 
  Cooldown = 30,
  CheckFirst = function()
    return not a.InExecute or a.VictoriousDuration > 0
  end
})

c.AddOptionalSpell("Impending Victory", "for Heals, Optional", {
  Melee = true,
  Cooldown = 30,
  CheckFirst = victoryHealable,
})

c.AddSpell("Victory Rush", nil, {
  Melee = true,
  CheckFirst = function()
    return a.VictoriousDuration > 0
  end,
})

c.AddSpell("Victory Rush", "for Heals", {
  Melee = true,
  CheckFirst = function()
    return a.VictoriousDuration > 0 and victoryHealable()
  end,
})

c.AddOptionalSpell("Victory Rush", "for Heals, Optional", {
  Melee = true,
  CheckFirst = function()
    return a.VictoriousDuration > 0 and victoryHealable()
  end,
})

--TALENT:
--Instantly heals you for 10% of your maximum health, 
--and an additional 20% over 5 sec.  Usable while stunned.
--c.AddOptionalSpell("Enraged Regeneration", nil, {
--  NoGCD = true,
--  NoPowerCheck = true,
--  Cooldown = 60,
--  CheckFirst = function()
--    return a.Enraged and c.GetHealthPercent("player") < 80
--  end,
--})

--TALENT:
--Every 30 rage that you spend reduces the remaining cooldown 
--of several of your abilities by 1 sec:
--  Avatar
--  Bloodbath
--  Bladestorm
--  Storm Bolt
--  Shockwave
--  Dragon Roar
--  Mocking Banner
--  Heroic Leap
--  Recklessness
--  Die by the Sword
c.AddOptionalSpell("Anger Management", nil, {
  NoGCD = true,
})

--TALENT:  Associated Buff: Avatar
--Transform into a colossus for 24 sec, dealing 20% 
--increased damage and removing all roots and snares.
c.AddOptionalSpell("Avatar", nil, {
  NoGCD = true,
  Cooldown = 180,
  CheckFirst = function()
    local reckCd = c.GetCooldown("Recklessness", true, 180)
    return reckCd > 42 or reckCd < 6
  end
})

c.AddOptionalSpell("Sunder Armor", nil, {
  CheckFirst = function()
    return not c.IsSolo()
      and (c.GetDebuffStack(c.ARMOR_DEBUFFS) < 3
        or c.GetDebuffDuration(c.ARMOR_DEBUFFS) < 3)
  end
})

c.AddInterrupt("Pummel")
c.AddInterrupt("Disrupting Shout")

-------------------------------------------------------------------------- Arms
local rendUpTime = 0

local function getRendUptime(time)
   if rendUpTime == 0 then
      rendUpTime = GetTime()
    return true
   elseif (GetTime() - rendUpTime) >= 17 then
      rendUpTime = 0
    return true
   else
      return false
   end

end

-- A vicious strike that deals 225% Physical damage and costs 20 rage.
-- Costs 20 rage.
c.AddSpell("Mortal Strike", nil, {
  Melee = true,
  Cooldown = 6,
  CheckLast = function()
     return not a.InExecute and a.MSIsUsable
  end
})

-- Wounds the target, causing (579.6% of Attack Power) bleed damage over 18 seconds and abilities
-- final burst (222.5% of Attack Power) bleed damage when the effect expires.
-- Costs 5 rage. 
c.AddSpell("Rend", nil, { 
  CheckFirst = function()
    return not c.HasMyDebuff("Rend")
  end
})
c.AddOptionalSpell("Recklessness", "for Arms", {
  CheckFirst = function(z)
    z.FlashSize = nil
    
    if c.IsSolo() or c.WearingSet(4, "DpsT14") then
      return true
    end
    
    if not a.InExecute then
      z.FlashSize = s.FlashSizePercent() / 2
    end
    return a.Bloodbath --or hasSmashFor(5)
  end
})

c.AddOptionalSpell("Bloodbath", "for Arms", {
  NoGCD = true,
  CheckFirst = function()
    return a.Rage >= 70 and a.smashPending > 0
  end
})

c.AddOptionalSpell("Berserker Rage", "for Arms", {
  NoGCD = true,
  CheckFirst = function()
    return not a.Enraged and a.EmptyRage >= 10
  end
})

c.AddOptionalSpell("Heroic Strike", "for Arms", {
  NoGCD = true,
  CheckFirst = function()
    return (a.EmptyRage < 15 and not a.InExecute)
      or (a.smashPending > 0 and a.Rage > 60 and c.WearingSet(2, "DpsT16"))
  end
})

c.AddOptionalSpell("Sweeping Strikes", nil, {
  NoGCD = true,
})

-- Hurls your weapon at an enemy, causing 60% Physical damage and stunning for 4 seconds.
-- Deals quadruple damage to targets permanently immune to stuns.
c.AddSpell("Storm Bolt", "for Arms", {
  Cooldown = 30,  
  CheckFirst = function()
    return a.smashPending > 0
  end
})

-- Roar explosively, dealing (156% of Attack Power) damage to all enemies within 8 yards
-- and knocking them back and down for 0.50 seconds.  Damage ignores all armor and is 
-- always a critical strike.
c.AddSpell("Dragon Roar", "for Arms", {
  Melee = true,
  Cooldown = 60,
})

-- Smashes a target for 150% Physical damage and causes your attacks to bypass all of their 
-- armor for 6 seconds.  Bypasses less armor on players.
c.AddSpell("Colossus Smash", "for Arms", {
  Cooldown = 20,
  Melee = true,
  Override = function()
    return a.CSIsUsable
  end
})

-- Attempt to finish off a wounded foe, causing 320% Physical damage. 
-- Only usable on enemies that have less than 20% health.
-- Costs 30 Rage
c.AddSpell("Execute", "for Arms", {
  CheckFirst = function()
    return a.InExecute or a.freeExecute
  end
})

-- Slam an opponent, causing 150% Physical damage.  Each consecutive use of Slam increases
-- the damage dealt by 50% and the Rage cost by 100%, stacking up to 2 times for 2 sec.
-- Costs 10 Rage.
c.AddSpell("Slam", nil, {
  CheckFirst = function(z)
      if c.GetBuffStack("Slam") == 1 then
       z.FlashSize = s.FlashSizePercent() * 0.75
    elseif c.GetBuffStack("Slam") == 2 then
       z.FlashSize = s.FlashSizePercent() * 1.5
    end
    return true
  end
})

c.AddSpell("Thunder Clap", nil, {
  Melee = true,
})

-- In a whirlwind of steel you attack all enemies within 8 yards, 
-- causing 100% Physical damage with your main-hand weapon to each enemy.
-- Costs 20 Rage.
c.AddSpell("Whirlwind", "for Arms", {
  Melee = true,
  CheckFirst = function()
    return a.Rage >= 20
  end,
})

-------------------------------------------------------------------------- Fury
-- 

local function shouldFlashBloodthirst()
  return (not a.InExecute)
    and not c.IsCasting("Bloodthirst")
end

local function getRagingBlowStacks(noGCD)
  local stacks = c.GetBuffStack("Raging Blow!", noGCD)
  if c.IsCasting("Enrage") then
    stacks = stacks + 1
  end
  if c.IsCasting("Raging Blow") then
    stacks = stacks - 1
  end
  local str = tostring(math.min(math.max(0, stacks), 2))
  return math.min(math.max(0, stacks), 2) 
end

c.AddOptionalSpell("Recklessness", "for Fury", {
  NoGCD = true,
  Cooldown = 180,
})

c.AddOptionalSpell("Bloodbath", "for Fury", {
  NoGCD = true,
  Cooldown = 60,
})

c.AddOptionalSpell("Berserker Rage", "for Fury", {
  NoGCD = true,
  CheckFirst = function()
    if c.HasTalent("Unquenchable Thirst") then
       return not a.Enraged
    else
       return not a.Enraged and getRagingBlowStacks(true) < 2
    end
--    local blowStacks = getRagingBlowStacks(true)
--    return (not a.Enraged and (blowStacks < 2 or a.InExecute))
--      or (c.GetBuffDuration("Recklessness", true, true) > 10 
--        and blowStacks == 0)
  end
})

c.AddOptionalSpell("Heroic Strike", "for Fury", {
  NoGCD = true,
  CheckFirst = function()
    return a.EmptyRage <= 10
      or (not a.InExecute and a.Rage >= 40 and a.smashPending > 0)  --a.Smash > 0)
  end
})

-- When Raging Blow is available, less than two charges
c.AddSpell("Raging Blow", nil, {
  Override = function(z)
    z.FlashSize = nil
    local stacks = getRagingBlowStacks()
    if stacks == 2 then
      z.FlashSize = s.FlashSizePercent()
      return true
    end
    
    if stacks == 1 then
      z.FlashSize = s.FlashSizePercent() / 2
      return true
    end
  end
})

-- When Raging Blow has two charges
c.AddSpell("Raging Blow", "Prime", {
  CheckFirst = function()
    return not a.InExecute
      and getRagingBlowStacks() == 2
  end
})

c.AddSpell("Raging Blow", "AoE", {
  CheckFirst = function()
    local now = GetTime()
    if c.IsCasting("Raging Blow") or now - a.CleaverDumpPending < .8 then
      return false
    end
    
    local stack = c.GetBuffStack("Meat Cleaver")
    if GetTime() - a.CleaverPending < .8 then
      stack = stack + 1
    end
    return stack >= 3
  end
})

c.AddSpell("Bloodthirst", nil, {
  CheckFirst = function()
    if c.HasTalent("Unquenchable Thirst") then
       return not a.Enraged
    elseif not a.Enraged then
       return a.Rage < 80
    else
       return select(1,s.UsableSpell("Bloodthirst")) and getRagingBlowStacks() < 2
    end
  end
})

c.AddSpell("Wild Strike", nil, {  
  NoPowerCheck = true,
  CheckFirst = function()
    return c.HasBuff("Bloodsurge") or a.Rage >= 90
  end
})

c.AddSpell("Wild Strike", "under Bloodsurge", {  
  NoPowerCheck = true,
  CheckFirst = function()
    if c.HasBuff("Bloodsurge") then
      return a.Rage >= 10
    else
      return a.Rage >= 80
    end
  end
})

--10122014 rdh: Removed reference to Smash
--Deals 1,371 damage to all enemies within 8 yards and knocking 
--them back and down for .5 sec.  Damage ignores all armor and is 
--always a critical strike.
c.AddSpell("Dragon Roar", "for Fury", {
  Melee = true,
  CheckFirst = function()
    return (not c.HasTalent("Bloodbath") or a.Bloodbath)
  end
})

--10122014 rdh: Changed rage check from 90 to 30
--   Removed a.smash since Colossus Smash has been removed.
--Attempt to finish off a wounded foe, causing
--3,237 Physical damage with main-hand and 969 
--Physical damage with off-hand.  Only usable on 
--enemies with less than 20% health.
c.AddSpell("Execute", "for Fury", {
  CheckFirst = function()
    return a.InExecute or a.freeExecute
  end
})

--10122014 rdh: Changed Cooldown from 30 to 6
--  Removed the Smash return, not necessary
--Throw weapon at the enemy, causing 510 Physical damage.
--Generates high threat.
--Note: What does Smash have to do with Heroic Throw?
c.AddSpell("Heroic Throw", "for Fury", {
  Cooldown = 6,
})

--In a whirlwind of steel you attack all enemies within 8 yards 
--causing 815 Physical damage with main-hand and 238 Physical 
--damage with off-hand to each enemy.  Dealing damage with Whirlwind 
--increases the number of targets that your Raging Blow hits by 1.
--Note:  What is the Override return?
--stacking up to 3.
c.AddSpell("Whirlwind", "for Fury", {
  Melee = true,
  Override = function()
    return a.Rage >= 30
  end,
})

-------------------------------------------------------------------- Protection
local function shouldDumpProt()
  return c.HasBuff("Ultimatum", true) 
    or c.HasBuff("Incite", true)
    or (c.HasGlyph("Incite") and c.IsCasting("Demoralizing Shout"))
    or (a.EmptyRage < 20 and not c.IsTanking())
    or c.InDamageMode()
end

c.AddOptionalSpell("Defensive Stance", nil, {
  Type = "form",
})

c.AddSpell("No Mitigation if Victory Available", nil, {
  ID = 0,
  RunFirst = function(z)
    z.ID = c.HasTalent("Impending Victory")
      and c.GetID("Impending Victory")
      or c.GetID("Victory Rush")
  end,
  ShouldHold = function()
    return true
  end
})

--10122014 rdh: Added Cooldown
--Demoralizes all enemies within 10 yards, reducing the
--damage they do to you by 20% for 8 sec.
c.AddOptionalSpell("Demoralizing Shout", nil, {
  NoGCD = true,
  Melee = true,
  Cooldown = 60,
  IsUp = function()
    return c.HasMyDebuff("Demoralizing Shout", true)
  end,
  CheckFirst = function(z)
    return not z.IsUp()
  end,
})

--10122014 rdh:  Removed for Prot,and added Cooldown
--Instantly heals for 10% of your maximum health, and an additional
--20% over 5 sec.  Usable while stunned.
--c.AddOptionalSpell("Enraged Regeneration", "for Prot", {
c.AddOptionalSpell("Enraged Regeneration", nil, {
  NoGCD = true,
  NoPowerCheck = true,
  Cooldown = 60,
  CheckFirst = function()
    return a.Enraged
  end,
  ShouldHold = function()
    return c.GetHealthPercent("player") > 90
  end,
})

--Reduces all damage taken by 40% for 8 sec.
c.AddOptionalSpell("Shield Wall", "under 3 min", {
  NoGCD = true,
  Enabled = function()
    return not c.HasGlyph("Shield Wall")
  end,
})

--Reduces all damage taken by 40% for 8 sec.
c.AddOptionalSpell("Shield Wall", "3+ min", {
  NoGCD = true,
  Enabled = function()
    return c.HasGlyph("Shield Wall")
  end,
})

--10122014 rdh: Added cooldown
--Note: Confused about the return value.
--Increases Current and Maximum health by 30% for 15 sec.
c.AddOptionalSpell("Last Stand", nil, {
  NoGCD = true,
  Cooldown = 180,
  ShouldHold = function()
    return s.HealthPercent("player") > 70
  end
})

--10122014 rdh: Removed Tanking check and added Rage check
--Raise our shield blocking every melee attack for the next 6 seconds.
--These can be critical blocks.  Maximum 2 charges.
c.AddOptionalSpell("Shield Block", nil, {
  NoGCD = true,
  Melee = true,
  CheckFirst = function(z)
    if not (a.Rage > 60)
      and not c.HasBuff("Shield Barrier", true)
      and not c.InDamageMode() then
        z.FlashSize = s.FlashSizePercent() / 2
    end
    return a.Rage > 60
      and not c.HasBuff("Shield Barrier", true)
      and not c.InDamageMode()
  end,
})

--10122014 rdh: Removed the tanking check
--Raise our shield absorbing  899 damage for the next 6 seconds.
--Consumes up to 40 additional Rage to increase the damage taken.
--Note: WoD allows this on ALL Warriors. Confused how this works 
--      with Fury and Arms.
c.AddOptionalSpell("Shield Barrier", nil, {
  NoGCD = true,
  Cooldown = 1.38,
  CheckFirst = function(z)
--    return c.IsTanking()
--      and not c.HasBuff("Shield Barrier", true) 
--      and not c.HasBuff("Shield Block", true)
--      and not c.InDamageMode()
    if a.Rage > 60 
      and not c.HasBuff("Shield Barrier", true) 
      and not c.HasBuff("Shield Block", true)
      and not c.InDamageMode() then
        z.FlashSize = s.FlashSizePercent() / 2
    end
    return a.Rage > 60 
      and not c.HasBuff("Shield Barrier", true) 
      and not c.HasBuff("Shield Block", true)
      and not c.InDamageMode()
  end,
})

--10122014 rdh: Removed Rage check
--You go berserk, removing and granting immunity to Fear,
--Sap, and Incapacitate effects for 6 sec.
c.AddOptionalSpell("Berserker Rage", "for Prot", {
  NoGCD = true,
  Cooldown = 30,
  CheckFirst = function(z)
--    return a.EmptyRage > 10 and (c.IsTanking() or c.InDamageMode())
    if c.IsTanking() or c.InDamageMode() then
      z.FlashSize = s.FlashSizePercent() / 2
    end
    return c.IsTanking() or c.InDamageMode()
  end,
})

-- 10122014 rdh: Added the cooldown and Rage check
-- Instantly deals 105% Physical damage.  This ability is not on the global cooldown.
-- Costs 30 rage
c.AddOptionalSpell("Heroic Strike", "for Prot", {
  NoGCD = true,
  Cooldown = 1.5,
  CheckFirst = function()
    if c.HasBuff("Ultimatum") then
      return true
    elseif c.HasTalent("Unyielding Strikes") then
      local dStacks = c.GetBuffStack("Unyielding Strikes", noGCD)
      return dStacks == 6 and a.Rage > 60
         or dStacks == 5 and a.Rage > 65
         or dStacks == 4 and a.Rage > 70
         or dStacks == 3 and a.Rage > 75
         or dStacks == 2 and a.Rage > 80
         or dStacks == 1 and a.Rage > 85
    else
      return a.Rage > 90
    end     
  end,
})

--10122014 rdh: Obsolete for Wod
--c.AddOptionalSpell("Cleave", "for Prot", {
--  NoGCD = true,
--  CheckFirst = function()
--    return c.AoE and shouldDumpProt()
--  end,
--})

c.AddTaunt("Taunt", nil, {
  NoGCD = true,
  Cooldown = 6,
  CheckFirst = function(z)
    if (c.HasTalent("Vigilance") and c.HasBuff("Vigilance")) or c.GetCooldown("Taunt") < 1 then
      z.FlashSize = s.FlashSizePercent() / 2
    end
    return (c.HasTalent("Vigilance") and c.HasBuff("Vigilance")) or c.GetCooldown("Taunt") < 1 
  end,
      
})

--10122014 rdh: Removed Weakend_Blows and changed cooldown from 6 to 5.54
c.AddSpell("Thunder Clap", "for Debuff", {
  Melee = true,
  Cooldown = 5.54,
  CheckFirst = function()
--    local debuffNeeded = not c.HasDebuff(c.WEAKENED_BLOWS_DEBUFFS)
    local damageMode = c.InDamageMode()
    if c.AoE then
--      return debuffNeeded or damageMode
      return damageMode
    else
--      return debuffNeeded and not damageMode
            return not damageMode
    end
  end,
})

c.AddSpell("Thunder Clap", "for Refresh", {
  Melee = true,
  Cooldown = 5.54,
  CheckFirst = function()
--    return c.GetDebuffDuration(c.WEAKENED_BLOWS_DEBUFFS) < 4 or c.AoE
       return c.AoE
  end,
})

c.AddSpell("Shield Slam", nil, {
  Cooldown = 6,
  CheckFirst = function() 
      return c.GetCooldown("Shield Slam") == 0
--    if c.HasBuff("Sword and Board") then
--      return a.EmptyRage > 25 and c.GetCooldown("Shield Slam") < 1
--    else
--      return a.EmptyRage > 20 and c.GetCooldown("Shield Slam") < 1
--    end
--    return false
  end,
})

c.AddOptionalSpell("Bloodbath", "for Prot", {
  NoGCD = true,
  Cooldown = 60,
  CheckFirst = function()
     return c.GetCooldown("Bloodbath")  == 0
  end,
})

c.AddSpell("Revenge", nil, {
  Melee = true,
--  GetDelay = function()
    CheckFirst = function()
      return c.GetCooldown("Revenge") == 0
  end,
})

c.AddSpell("Delay for Prot Rage Generator", nil, {
  ID = "Battle Shout",
  IsMinDelayDefinition = true,
  GetDelay = function()
    return math.min(
        c.GetCooldown("Shield Slam", false, 6), 
        c.GetCooldown("Revenge", false, 9),
        c.GetCooldown("Battle Shout", false, 60)), 
      c.InDamageMode() and .2 or .5
  end,
})

--Attempt to finish off a wounded foe, causing
--3,237 Physical damage with main-hand and 969 
--Physical damage with off-hand.  Only usable on 
--enemies with less than 20% health.
c.AddSpell("Execute", "for Prot", {
  CheckFirst = function()
    return a.freeExecute or a.InExecute
  end
})

c.AddSpell("Devestate", nil, {
  Melee = true, -- not sure why this is necessary.
  CheckFirst = function()
     return c.GetBuffStack("Unyielding Strikes", noGCD) < 6
  end,
})

c.AddSpell("Devestate", "for Debuff unless AoE", {
  Melee = true, -- not sure why this is necessary.
  CheckFirst = function()
    local stack = c.GetDebuffStack(c.ARMOR_DEBUFFS)
    local duration
    if c.IsCasting("Devestate") then
      stack = stack + 1
      duration = 30
    else
      duration = c.GetDebuffDuration(c.ARMOR_DEBUFFS)
    end
    return (stack < 3 or duration < 3) and not c.AoE
  end,
})
